#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

#################################################################################
# Created       : Martin Foster
# Modified      : 19-Mar-2007
#################################################################################
#
# Transfer - Generate SQL output in order to transfer from one site to another
# Copyright (C) 2000-2007  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 4-3526 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Source;						# Source database


#################################################################################
# Data Members
#################################################################################
my $source;							# Database handle
my $dest;							# Destination database
my $old;							# Site name to transfer
my $new;							# New site transfer

my $capture;							# Captured input
my $elem;							# Element to display
my $mode;							# Mode

my %active;							# Active components

#################################################################################
# Program Area
#################################################################################

	# Make sure schema is specified
	if ($old = shift(@ARGV))
	{
		# Additional schema specification
		if ($new = shift(@ARGV))
		{
			# Establish defaults
			%active = (
				Font        => 'off',
				Gummy       => 'on',
				Genre       => 'off',
				Fortune     => 'off',
				User        => 'on',
				Access      => 'off',
				Puppet      => 'on',
				Gallery     => 'off',
				Offline     => 'off',
				Mail        => 'off',
				Destroy     => 'off'
			 );

			# Loop and establish settings
			do
			{
				# Clear the screen
				system "clear";

				# Display every element
				foreach $elem (keys(%active))
				{
					# Toggle?
					if (lc($capture) eq lc($elem)) 
					{
						# Switch current value
						$mode = 'on'  if ($active{$elem} eq 'off');
						$mode = 'off' if ($active{$elem} eq 'on');

						# Set mode
						$active{$elem} = $mode;
					}

					# Display to screen
					print $elem . "\t.\t.\t.\t" . $active{$elem} . "\n";
				}

				# Notice
				print "\n\nNOTE: Destroy is used to remove previous information from\n" 
				    . "the table prior to transfering the information.\n\n";

				# Capture for next pass
				$capture = Capture('Done');

			# Until the capture is proper
			} while ($capture !~ /Done/i);



			# Initialize handles
			$dest   = new Ethereal::Database();
			$source = new Ethereal::Source();

			# Connect and Fetch
			$dest->Connect();
			$source->Connect();
	
			# Set search paths
			$dest->Write("SET SEARCH_PATH TO $new,public");
			$source->Write("SET SEARCH_PATH TO $old,public");

			# Generate file
			Generate($source, $dest);

			# Inform
			print "Completed generation of SQL file for $new from $old\n\n";
		}
		else
		{
			# Simple useage instead of error
			print "Ethereal Realms transfer command-line interface:\n";
			print "Usage: ./xfer.pl {read schema} {write schema}\n\n";
		}


	}

	else
	{
		# Simple useage instead of error
		print "Ethereal Realms transfer command-line interface:\n";
		print "Usage: ./xfer.pl {read schema} {write schema}\n\n";
	}



#################################################################################
# Sub-Routines
#################################################################################


#####################
# Capture Input
#
# User input is critical to many parts of the configuration process.

sub Capture
{
	#####################
	# Data members
	my $default	= shift;				# Default value

	my $capture;						# Captured line
	my $display;						# Displayed default
	
	#####################
	# Program area

	# Always assigned
	while (!defined($capture))
	{
		# Ensure definition
		$display = (defined($default)) ? $default : '';

		# Prompt human for question
		print " <Type in name to toggle> [" . $display . "] ";


		# Await human input
		$capture = <STDIN>;


		# Assign default if need be
		$capture = ($capture =~ /^\n/) ? $default : $capture;
	}
	
	# Carriage return
	chomp($capture);

	# Return
	return $capture;
}



#####################
# Short Name
#
# Generates short names required for new versions of Ethereal Realms.

sub Short
{
	#####################
	# Data members

	my $full    = shift;					# Full name
	my $short   = shift;					# Short name


	#####################
	# Program area

	# Generate short name
	$short  =  lc($full);
	$short  =~ s/-/_/gs;
	$short  =~ s/\s/_/gs;
	$short  =~ s/\W//gs;

	# Return value
	return $short;
}



#####################
# Generate
#
# Grabs from the old and hands off the the new.  Nothing more difficult then that
# and should be faily easy to adapt to older scripts.

sub Generate
{
	#####################
	# Data members
	my $data    = shift;					# Database handle
	my $write   = shift;					# Writing database handle
	my $active  = shift;					# Active elements

	my $file;						# Filename
	my $res;						# Result handle
	my $seq;						# Sequence number
	my $statement;						# Query

	#####################
	# Program area

	#
	#
	# Font
	if ($active{'Font'} eq 'on')
	{
		$data->Pull(\$statement, "SELECT
			  FontName         AS \"FontName\",
			  FontType         AS \"FontType\",
			  FontCode         AS \"FontCode\"
			FROM Font
			WHERE FontType <> 'size'
			ORDER BY FontType, FontName");

		# Title
		print "\n\n--\n-- FONT\n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Font WHERE FontType <> 'size')")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Font (FontName,FontType,FontCode) VALUES(?,?,?)",
				$res->{'FontName'},
				$res->{'FontType'},
				$res->{'FontCode'}
			 );
		}

		# End
		$statement->finish();
	}


	#
	#
	# Gummies
	if ($active{'Gummy'} eq 'on')
	{
		$data->Pull(\$statement, "SELECT
			  GummyName        AS \"GummyName\",
			  GummyClass       AS \"GummyClass\",
			  GummyPath        AS \"GummyPath\",
			  GummyIdent       AS \"GummyIdent\",
			  GummyMacro       AS \"GummyMacro\"
			FROM Gummy
			WHERE RealmName=?
			  AND GummyIdent IS NULL
			ORDER BY GummyClass, GummyName", 'system');

		# Title
		print "\n\n--\n-- GUMMY\n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Gummy WHERE RealmName='system'")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Gummy (GummyName,GummyClass,GummyPath,GummyIdent,GummyMacro) VALUES(?,?,?,?,?)",
				$res->{'GummyName'},
				$res->{'GummyClass'},
				$res->{'GummyPath'},
				$res->{'GummyIdent'},
				$res->{'GummyMacro'}
			 );
		}

		# End
		$statement->finish();
	}


	#
	#
	# Genres
	if ($active{'Genre'} eq 'on')
	{
		$data->Pull(\$statement, "SELECT
			  GenreName        AS \"GenreName\",
			  GenreType        AS \"GenreType\",
			  GenreSub         AS \"GenreSub\",
			  GenreDescription AS \"GenreDescription\"
			FROM Genre
			ORDER BY GenreName GenreType, GenreSub");

		# Title
		print "\n\n--\n-- GENRE\n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Genre")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Genre (GenreName,GenreType,GenreSub,GenreDescription) VALUES(?,?,?,?)",
				$res->{'GenreName'},
				$res->{'GenreType'},
				$res->{'GenreSub'},
				$res->{'GummyIdent'},
				$res->{'GenreDescription'}
			 );
		}

		# End
		$statement->finish();
	}


	#
	#
	# Fortunes
	if ($active{'Fortune'} eq 'on')
	{
		$data->Pull(\$statement, "SELECT
			  FortuneNum     AS \"GenreName\",
			  FortuneType    AS \"GenreType\",
			  FortuneMsg     AS \"GenreSub\",
			FROM Fortune
			ORDER BY FortuneNum, FortuneType");

		# Title
		print "\n\n--\n-- FORTUNE\n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Fortune")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Fortune (FortuneType,FortuneMsg) VALUES(?,?)",
				$res->{'FortuneType'},
				$res->{'FortuneMsg'}
			 );
		}

		# End
		$statement->finish();
	}


	#
	#
	# Users
	if ($active{'User'} eq 'on')
	{
		$data->Pull(\$statement, "SELECT
				PuppeteerLogin      AS \"PuppeteerLogin\",
				PuppeteerPassword   AS \"PuppeteerPassword\",
				PuppeteerName       AS \"PuppeteerName\",
				PuppeteerEMail      AS \"PuppeteerEMail\",
				PuppeteerVerified   AS \"PuppeteerVerified\",
				PuppeteerAdminPrivs AS \"PuppeteerAdminPrivs\",
				PuppeteerSuperPrivs AS \"PuppeteerSuperPrivs\",
				PuppeteerModPrivs   AS \"PuppeteerModPrivs\",
				PuppeteerHome       AS \"PuppeteerHome\",
				PuppeteerHomeSpan   AS \"PuppeteerHomeSpan\"
			FROM Puppeteer
			WHERE PuppeteerBan <> 'yes'
			ORDER BY PuppeteerLogin");	

		# Title
		print "\n\n--\n-- PUPPETEER\n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Puppeteer WHERE PuppeteerLogin <> 'system'")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Puppeteer (PuppeteerLogin, PuppeteerPassword, PuppeteerName, PuppeteerEMail, PuppeteerVerified, PuppeteerAdminPrivs, PuppeteerSuperPrivs, PuppeteerModPrivs, PuppeteerHome, PuppeteerHomeSpan) VALUES(?,?,?,?,?,?,?,?,?,?)",
				$res->{'PuppeteerLogin'},
				$res->{'PuppeteerPassword'},
				$res->{'PuppeteerName'},
				$res->{'PuppeteerEMail'},
				$res->{'PuppeteerVerified'},
				$res->{'PuppeteerAdminPrivs'},
				$res->{'PuppeteerSuperPrivs'},
				$res->{'PuppeteerModPrivs'},
				$res->{'PuppeteerHome'},
				$res->{'PuppeteerHomeSpan'}
			 );
		}

		# End
		$statement->finish();
	}


	#
	#
	# Realm Supervisor
	if ($active{'Access'} eq 'on')
	{
		$data->Pull(\$statement, "SELECT
			  RealmName        AS \"RealmName\",
			  PuppeteerLogin   AS \"PuppeteerLogin\",
			  RealmACL         AS \"RealmACL\"
			FROM RealmSupervisor
			ORDER BY RealmName, PuppeteerLogin");

		# Title
		print "\n\n--\n-- REALM SUPERVISOR\n--\n";

		# Cleanup
		$write->Quick("DELETE FROM RealmSupervisor")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO RealmSupervisor (RealmName, PuppeteerLogin, RealmACL) VALUES(?,?,?)",
				$res->{'RealmName'},
				$res->{'PuppeteerLogin'},
				$res->{'RealmACL'}
			 );
		}

		# End
		$statement->finish();
	}


	#
	#
	# Puppet
	if ($active{'Puppet'} eq 'on')
	{
		$data->Pull(\$statement, "SELECT
			  PuppetName      AS \"PuppetName\",
			  PuppetPool      AS \"PuppetPool\",
			  PuppeteerLogin  AS \"PuppeteerLogin\",
			  PuppetDefault   AS \"PuppetDefault\",
			  PuppetBio       AS \"PuppetBio\",
			  PuppetTag       AS \"PuppetTag\",
			  PuppetTitle     AS \"PuppetTitle\",
			  PuppetGummy     AS \"PuppetGummy\",
			  PuppetPic       AS \"PuppetPic\",
			  PuppetPicHeight AS \"PuppetPicHeight\",
			  PuppetPicWidth  AS \"PuppetPicWidth\",
			  PuppetPicSize   AS \"PuppetPicSize\"
			FROM Puppet
			WHERE PuppetPool NOT IN ('banned')
			ORDER BY PuppeteerLogin, PuppetName");

		# Title
		print "\n\n--\n-- PUPPETS\n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Puppet")
		 if ($active{'Destroy'} eq 'on');
	
		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Puppet (PuppetName, PuppetPool, PuppeteerLogin, PuppetShort, PuppetDefault, PuppetBio, PuppetTag, PuppetTitle, PuppetGummy, PuppetPic, PuppetPicHeight, PuppetPicWidth, PuppetPicSize) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?)",
				  $res->{'PuppetName'},
				  $res->{'PuppetPool'},
				  $res->{'PuppeteerLogin'},
				  Short($res->{'PuppetName'}),
				  $res->{'PuppetDefault'},
				  $res->{'PuppetBio'},
				  $res->{'PuppetTag'},
				  $res->{'PuppetTitle'},
				  $res->{'PuppetGummy'},
				  $res->{'PuppetPic'},
				  $res->{'PuppetPicHeight'},
				  $res->{'PuppetPicWidth'},
				  $res->{'PuppetPicSize'}
			   );
		}

		# End
		$statement->finish();
	}


	#
	#
	# Galleries
	if ($active{'Gallery'} eq 'on')
	{
		# Pull all information
		$data->Pull(\$statement, "SELECT
			  GalleryID          AS \"GalleryID\",
			  PuppeteerLogin     AS \"PuppeteerLogin\",
			  PuppeteerPenName   AS \"PuppeteerPenName\",
			  GalleryName        AS \"GalleryName\",
			  GalleryRating      AS \"GalleryRating\",
			  GalleryDescription AS \"GalleryDescription\",
			  GalleryWork        AS \"GalleryWork\",
			  GalleryThumbnail   AS \"GalleryThumbnail\",
			  GalleryComments    AS \"GalleryComments\",
			  GallerySort        AS \"GallerySort\",
			  GalleryType        AS \"GalleryType\",
			  GallerySub         AS \"GallerySub\",
			  GalleryCreated     AS \"GalleryCreated\",
			  GalleryUpdated     AS \"GalleryUpdated\"
			FROM Gallery
			ORDER BY PuppeteerLogin, GalleryName");

		# Title
		print "\n\n--\n-- GALLERIES \n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Gallery")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Gallery (GalleryID,PuppeteerLogin,PuppeteerPenName,GalleryName,GalleryRating,GalleryDescription,GalleryWork,GalleryThumbnail,GalleryComments,GallerySort,GalleryType,GallerySub,GalleryCreated,GalleryUpdated) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
				  $res->{'GalleryID'},
				  $res->{'PuppeteerLogin'},
				  $res->{'PuppeteerPenName'},
				  $res->{'GalleryName'},
				  $res->{'GalleryRating'},
				  $res->{'GalleryDescription'},
				  $res->{'GalleryWork'},
				  $res->{'GalleryThumbnail'},
				  $res->{'GalleryComments'},
				  $res->{'GallerySort'},
				  $res->{'GalleryType'},
				  $res->{'GallerySub'},
				  $res->{'GalleryCreated'},
				  $res->{'GalleryUpdated'}
			 );
		}

		# End
		$statement->finish();


		# ID numbers
		# Pull the value
		($seq) = $data->DataGet("SELECT CURRVAL('seqGallery')");

		# Set the value
		$write->Quick("SELECT SETVAL('seqGallery', ?)", $seq);



		#
		# Layers
		# Pull all information
		$data->Pull(\$statement, "SELECT
			  LayerID          AS \"LayerID\",
			  GalleryID        AS \"GalleryID\",
			  LayerShort       AS \"LayerShort\",
			  LayerFull        AS \"LayerFull\",
			  LayerDescription AS \"LayerDescription\",
			  LayerWork        AS \"LayerWork\",
			  LayerHighlight   AS \"LayerHighlight\",
			  LayerThumbnail   AS \"LayerThumbnail\",
			  LayerLevel       AS \"LayerLevel\",
			  LevelZero        AS \"LevelZero\",
			  LevelOne         AS \"LevelOne\",
			  LevelTwo         AS \"LevelTwo\",
			  LevelThree       AS \"LevelThree\",
			  LevelFour        AS \"LevelFour\",
			  LevelFive        AS \"LevelFive\"
			FROM Layer
			ORDER BY GalleryID, LayerID");

		# Title
		print "\n\n--\n-- LAYERS \n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Layer")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Layer (LayerID,GalleryID,LayerShort,LayerFull,LayerDescription,LayerWork,LayerHighlight,LayerLevel,LevelZero,LevelOne,LevelTwo,LevelThree,LevelFour,LevelFive) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
				  $res->{'LayerID'},
				  $res->{'GalleryID'},
				  $res->{'LayerShort'},
				  $res->{'LayerFull'},
				  $res->{'LayerDescription'},
				  $res->{'LayerWork'},
				  $res->{'LayerHighlight'},
				  $res->{'LayerLevel'},
				  $res->{'LevelZero'},
				  $res->{'LevelOne'},
				  $res->{'LevelTwo'},
				  $res->{'LevelThree'},
				  $res->{'LevelFour'},
				  $res->{'LevelFive'}
			 );
		}

		# End
		$statement->finish();


		# ID numbers
		# Pull the value
		($seq) = $data->DataGet("SELECT CURRVAL('seqLayer')");

		# Set the value
		$write->Quick("SELECT SETVAL('seqLayer', ?)", $seq);



		#
		# Works
		# Pull all information
		$data->Pull(\$statement, "SELECT
			  GalleryID       AS \"GalleryID\",
			  WorkID          AS \"WorkID\",
			  PuppeteerLogin  AS \"PuppeteerLogin\",
			  WorkName        AS \"WorkName\",
			  WorkLayer       AS \"WorkLayer\",
			  WorkDescription AS \"WorkDescription\",
			  WorkSort        AS \"WorkSort\",
			  WorkThumbnail   AS \"WorkThumbnail\",
			  WorkImageSize   AS \"WorkImageSize\",
			  WorkImageHeight AS \"WorkImageHeight\",
			  WorkImageWidth  AS \"WorkImageWidth\",
			  WorkImageView   AS \"WorkImageView\",
			  WorkImageFull   AS \"WorkImageFull\",
			  WorkText        AS \"WorkText\",
			  WorkPreserve    AS \"WorkPreserve\",
			  WorkTimestamp   AS \"WorkTimestamp\"
			FROM Work
			ORDER BY PuppeteerLogin, GalleryID, WorkID");

		# Title
		print "\n\n--\n-- WORKS \n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Work")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Work (GalleryID,WorkID,PuppeteerLogin,WorkName,WorkLayer,WorkDescription,WorkSort,WorkThumbnail,WorkImageSize,WorkImageHeight,WorkImageWidth,WorkImageView,WorkImageFull,WorkText,WorkPreserve,WorkTimestamp) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
				  $res->{'GalleryID'},
				  $res->{'WorkID'},
				  $res->{'PuppeteerLogin'},
				  $res->{'WorkName'},
				  $res->{'WorkLayer'},
				  $res->{'WorkDescription'},
				  $res->{'WorkSort'},
				  $res->{'WorkThumbnail'},
				  $res->{'WorkImageSize'},
				  $res->{'WorkImageHeight'},
				  $res->{'WorkImageWidth'},
				  $res->{'WorkImageView'},
				  $res->{'WorkImageFull'},
				  $res->{'WorkText'},
				  $res->{'WorkPreserve'},
				  $res->{'WorkTimestamp'}
			 );
		}

		# End
		$statement->finish();


		# ID numbers
		# Pull the value
		($seq) = $data->DataGet("SELECT CURRVAL('seqWork')");

		# Set the value
		$write->Quick("SELECT SETVAL('seqWork', ?)", $seq);


		#
		# Comment
		# Pull all information
		$data->Pull(\$statement, "SELECT
			  GalleryID        AS \"GalleryID\",
			  WorkID           AS \"WorkID\",
			  CommentID        AS \"CommentID\",
			  PuppetName       AS \"PuppetName\",
			  PuppeteerLogin   AS \"PuppeteerLogin\",
			  CommentTimestamp AS \"CommentTimestamp\",
			  CommentPost      AS \"CommentPost\",
			  CommentRating    AS \"CommentRating\",
			FROM Comment 
			ORDER BY PuppeteerLogin, GalleryID, WorkID, CommentID");

		# Title
		print "\n\n--\n-- COMMENTS \n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Comment")
		 if ($active{'Destroy'} eq 'on');

		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Comment (GalleryID,WorkID,PuppetName,PuppeteerLogin,CommentTimestamp,CommentPost) VALUES(?,?,?,?,?,?,?)",
				  $res->{'GalleryID'},
				  $res->{'WorkID'},
				  $res->{'PuppetName'},
				  $res->{'PuppeteerLogin'},
				  $res->{'CommentTimestamp'},
				  $res->{'CommentPost'},
				  $res->{'CommentRating'}
			 );
		}

		# End
		$statement->finish();
	}


	#
	#
	# Offline
	if ($active{'Offline'} eq 'on')
	{
		$data->Pull(\$statement, "SELECT
			  OfflineIDNumber        AS \"OfflineIDNumber\",
			  RealmName              AS \"RealmName\",
			  PuppetName             AS \"PuppetName\",
			  PuppeteerLogin         AS \"PuppeteerLogin\",
			  OfflineTimestamp       AS \"OfflineTimestamp\",
			  OfflineFullFormat      AS \"OfflineFullFormat\",
			  OfflineImagelessFormat AS \"OfflineImagelessFormat\",
			  OfflinePartialFormat   AS \"OfflinePartialFormat\"
			FROM Offline
			ORDER BY RealmName, OfflineIDNumber");

		# Title
		print "\n\n--\n-- OFFLINE POSTS \n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Offline")
		 if ($active{'Destroy'} eq 'on');
	
		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Offline (RealmName,PuppetName,PuppeteerLogin,OfflineTimestamp,OfflineFullFormat,OfflineImagelessFormat,OfflinePartialFormat) VALUES(?,?,?,?,?,?,?)",
				  $res->{'RealmName'},
				  $res->{'PuppetName'},
				  $res->{'PuppeteerLogin'},
				  $res->{'OfflineTimestamp'},
				  $res->{'OfflineFullFormat'},
				  $res->{'OfflineImagelessFormat'},
				  $res->{'OfflinePartialFormat'}
			   );
		}

		# End
		$statement->finish();
	}


	#
	#
	# Mail
	if ($active{'Mail'} eq 'on')
	{
		$data->Pull(\$statement, "SELECT
			  MailIDNumber  AS \"MailIDNumber\",
			  MailFrom      AS \"MailFrom\",
			  MailFromLogin AS \"MailFromLogin\",
			  MailFromRead  AS \"MailFromRead\",
			  MailRcpt      AS \"MailRcpt\",
			  MailRcptLogin AS \"MailRcptLogin\",
			  MailRcptDel   AS \"MailRcptDel\",
			  MailSubject   AS \"MailSubject\",
			  MailTimestamp AS \"MailTimestamp\",
			  MailRead      AS \"MailRead\",
			  MailContents  AS \"MailContents\"
			FROM Mail
			ORDER BY MailIDNumber");

		# Title
		print "\n\n--\n-- MAIL \n--\n";

		# Cleanup
		$write->Quick("DELETE FROM Mail")
		 if ($active{'Destroy'} eq 'on');
	
		# Loop and display
		while ($res = $statement->fetchrow_hashref())
		{
			# Display
			$write->Quick("INSERT INTO Mail (MailFrom,MailFromLogin,MailFromRead,MailRcpt,MailRcptLogin,MailRcptDel,MailSubject,MailTimestamp,MailRead,MailContents) VALUES(?,?,?,?,?,?,?,?,?,?)",
				  $res->{'MailFrom'},
				  $res->{'MailFromLogin'},
				  $res->{'MailFromRead'},
				  $res->{'MailRcpt'},
				  $res->{'MailRcptLogin'},
				  $res->{'MailRcptDel'},
				  $res->{'MailSubject'},
				  $res->{'MailTimestamp'},
				  $res->{'MailRead'},
				  $res->{'MailContents'}
			   );
		}

		# End
		$statement->finish();
	}
}
